package network;

import rmi.*;
import access.Database;
import access.DatabaseConstants;
import business.ConfigurationManager;



import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Hashtable;
import java.util.Vector;

public class ConfigurationServer extends UnicastRemoteObject implements DatabaseConstants, AuralizerInterface, ListenerInterface, ExecPlatformInterface
{

    NetworkInfo objNI=null;
    ConfServerInterface objCSI=null;
    Hashtable htListenerConnections = null;

    public ConfigurationServer() throws RemoteException
    {
        htListenerConnections = new Hashtable();
        //System.out.println("CONSTRUCTOR OF CONF SERVER");
    }

    public boolean initialise()
    {
        if((objNI=retrieveNetworkInfo())==null)
        {
            objNI = new NetworkInfo();
            return false;
        }
        else
        {
            //System.out.println("Initialise ELSE part");
            if(initialiseAuraliserServer() && initialiseExecPlatformServer() && initialiseListenerServer())
            {
               System.out.println("Configuation Server Initialsed");
                return true;
            }
            else
            {
                System.out.println("Configuration Server Not Initialised");
                return false;
            }
        }
    }

    public String[] getNetworkInfo()
    {
        return objNI.getNetworkInfo();
    }

    public boolean setNetworkInfo(String sIPAddress, String sPortNo)
    {
        objNI.setConfServerIPAddress(sIPAddress);
        objNI.setConfServerPortNo(sPortNo);

        System.out.println(objNI.getConfServerIPAddress());
        System.out.println(objNI.getConfServerPortNo());

        if(storeNetworkInfo()==true)
        {
            if(initialiseAuraliserServer() && initialiseExecPlatformServer() && initialiseListenerServer())
            {
                return true;
            }
            else
            {
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    public boolean establishConnection(String sIPAddress, String sPortNo)
    {
        try
        {
            objCSI = (ConfServerInterface)Naming.lookup("rmi://"+ sIPAddress + ":" + sPortNo +"/CONF_SERVER_EVENT_RECEIVER");
            htListenerConnections.put(sIPAddress.concat(sPortNo),objCSI);
            //System.out.println("Connection Established, Hastable of logged in Listeners updated");
            return true;
        }
        catch(Exception e)
        {
            e.printStackTrace();
            System.out.println("Exception at Cofiguration Server Interface Communicator");
            return false;
        }

    }

    public boolean storeNetworkInfo()
    {
        Database objDB = new Database();
        return objDB.writeObject(NETWORK_INFO+NETWORK_INFO_FILE_EXTENSION, objNI);
    }

    public NetworkInfo retrieveNetworkInfo()
    {
        Database objDB = new Database();
        try
        {
            NetworkInfo objNI = (NetworkInfo)objDB.readObject(NETWORK_INFO+NETWORK_INFO_FILE_EXTENSION);
            return objNI;
        }
        catch(Exception e)
        {
            System.out.println("Retrieval of Network Information Failed");
            return null;
        }
    }

    public boolean initialiseListenerServer()
    {


/*
        if(System.getSecurityManager() == null)
        {
            System.setSecurityManager(new RMISecurityManager());
        }
*/


          try
          {
            System.out.println(objNI.getConfServerIPAddress() + ":" + objNI.getConfServerPortNo());
            Naming.rebind("rmi://"+objNI.getConfServerIPAddress()+ ":" + objNI.getConfServerPortNo() +"/LISTENER_SERVER", this);
            //Naming.rebind("rmi//localhost:2099/LISTENER_SERVER", this);
            System.out.println("Listener Server initialised");
          }
          catch(Exception ce)
          {
            System.out.println("Error in Listener Server Registration");
            return false;
          }
        return true;
    }

    public boolean initialiseAuraliserServer()
    {

        /*if(System.getSecurityManager() == null)
        {
            System.setSecurityManager(new RMISecurityManager());
        } */

          try
          {
              //System.out.println("IP ADDRESS : "+ objNI.getConfServerIPAddress());
              //System.out.println("Port No : "+ objNI.getConfServerPortNo());
              //String sURL = new String("rmi://"+objNI.getConfServerIPAddress()+ ":" + objNI.getConfServerPortNo() +"/AURALISER_SERVER");
              //System.out.println(sURL);
              //System.out.println("rmi://localhost:1099/AURALISER_SERVER");
              //System.out.println("Equals : "+ sURL.equalsIgnoreCase("rmi://localhost:1099/AURALISER_SERVER"));
            Naming.rebind("rmi://"+objNI.getConfServerIPAddress()+ ":" + objNI.getConfServerPortNo() +"/AURALISER_SERVER", this);
              //Naming.rebind("rmi://localhost:1099/AURALISER_SERVER", this);
          }
          catch(Exception ce)
          {
            ce.printStackTrace();
              //System.out.println("  ERR_--"+ce.getLocalizedMessage());

            System.out.println("Error in Auraliser Server Registration");
            return false;
          }
        return true;

    }

    public boolean initialiseExecPlatformServer()
    {
        /*
        if(System.getSecurityManager() == null)
        {
            System.setSecurityManager(new RMISecurityManager());
        } */

          try
          {
            Naming.rebind("rmi://"+objNI.getConfServerIPAddress()+ ":" + objNI.getConfServerPortNo() +"/EXECPLATFORM_SERVER", this);
          }
          catch(Exception ce)
          {
            System.out.println("Error in ExecPlatform Server Registration");
            return false;
          }
        return true;

    }

    public boolean registerAuralizedProgram(String sProgramName, Hashtable htEventSoundMapping) throws RemoteException
    {

        return ConfigurationManager.registerAuralisedProgram(sProgramName, htEventSoundMapping);
    }
    public boolean unRegisterAuralizedProgram(String sProgramName) throws RemoteException
    {
        return ConfigurationManager.unRegisterAuralisedProgram(sProgramName);
    }
    public void receiveEvent(String sProgramName, EventInfo objEI) throws RemoteException
    {
        System.out.println("Event Received at Configuration Server");
        //System.out.println("CHECK EXEC PLATFORM");
        Vector vArray[] = ConfigurationManager.getListeners(sProgramName);
        if(vArray !=null)
        {
            multicast(vArray,sProgramName,objEI);
        }
        else
        {
            System.out.println("No Listeners to send Event");
        }

    }
    public Hashtable registerProgram(String sProgramName, String sIPAddress, String sPort) throws RemoteException
   {
       return ConfigurationManager.registerProgramForListener(sProgramName, sIPAddress, sPort);

   }
    public boolean unRegisterProgram(String sProgramName, String sIPAddress, String sPort) throws RemoteException
    {
      return ConfigurationManager.unRegisterProgramForListener(sProgramName, sIPAddress, sPort);
    }

    public boolean registerListener(String sIPAddress, String sPort) throws RemoteException
    {
       System.out.println("Listener Registration:  " + sIPAddress + " : " +sPort);
       return ConfigurationManager.registerListener(sIPAddress, sPort);
    }

    public boolean unRegisterListener(String sIPAddress, String sPort) throws RemoteException
    {
       return ConfigurationManager.unRegisterListener(sIPAddress, sPort);
    }

    public Vector loginListener(String sIPAddress, String sPort) throws RemoteException
    {

        System.out.println("Listener Logging In : "  + sIPAddress + " : " +sPort);
        Vector vProgramList=ConfigurationManager.login(sIPAddress, sPort);
        if(vProgramList == null)
        {
            System.out.println("No Programs registered at Configuration Server");
        }
        if(this.establishConnection(sIPAddress, sPort) == false)
        {
            System.out.println("Establish Connection back to listener : Failed");
            return null;
        }
        else
        {
            System.out.println("Establish Connection back to Listener : Success");
            return vProgramList;
        }

    }

    public boolean logoutListener(String sIPAddress, String sPort) throws RemoteException
    {
        System.out.println("Listener Logs out : "  + sIPAddress + " : " +sPort);
        htListenerConnections.remove(sIPAddress.concat(sPort));
        return ConfigurationManager.logout(sIPAddress, sPort);

    }

    public void multicast(Vector[] vListenerInfo, String sProgramName, EventInfo objEI)
    {
        for(int iLoop=0; iLoop < vListenerInfo[0].size(); iLoop++)
        {
            //System.out.println("size="+vListenerInfo[0].size());
            //System.out.println("IP="+(String)vListenerInfo[0].get(iLoop)+"  port= "+(String)vListenerInfo[1].get(iLoop));
            String sName=((String)vListenerInfo[0].get(iLoop)).concat((String)vListenerInfo[1].get(iLoop));
             boolean bAvailable=((Boolean)vListenerInfo[2].get(iLoop)).booleanValue();
           if(bAvailable == true)
            {
                objCSI = (ConfServerInterface)htListenerConnections.get(sName);
                //System.out.println("CSI object  = "+objCSI);
                try
                {
                    objCSI.receiveEvent(sProgramName, objEI);
                }
                catch(RemoteException re)
                {
                    System.out.println("Remote Exception while sending events to Listener");
                    re.printStackTrace();

                }
                catch(Exception e)
                {
                    System.out.println("Exception at Configuration Server : NOT Remote Exception");
                    System.out.println("---------------------------------------------");
                    System.out.println("OBJECT=  "+objCSI);
                    System.out.println("---------------------------------------------");
                    e.printStackTrace();
                }
             }
        }

    }

   }

